"use strict";Object.defineProperty(exports,"__esModule",{value:!0}),exports.default=void 0;var _Validator=_interopRequireDefault(require("./Validator")),_util=require("./util"),_qs=_interopRequireDefault(require("qs"));function _interopRequireDefault(a){return a&&a.__esModule?a:{default:a}}function _defineProperty(a,b,c){return b in a?Object.defineProperty(a,b,{value:c,enumerable:!0,configurable:!0,writable:!0}):a[b]=c,a}var UNPROCESSABLE_ENTITY=422;class BaseProxy{/**
     * Create a new BaseProxy instance.
     *
     * @param {string} endpoint   The endpoint being used.
     * @param {Object} parameters The parameters for the request.
     */constructor(a,b){this.endpoint=a,this.parameters=b}/**
     * Get axios instance
     * @return {*}
     */get $http(){return BaseProxy.$http}/**
     * Method used to fetch all items from the API.
     *
     * @returns {Promise} The result in a promise.
     */all(){return this.submit("get","/".concat(this.endpoint))}/**
     * Method used to fetch a single item from the API.
     *
     * @param {string|int} id The given identifier.
     *
     * @returns {Promise} The result in a promise.
     */find(a){return this.submit("get","/".concat(this.endpoint,"/").concat(a))}/**
     * Send a POST request to the given URL.
     *
     * @param {Object} item
     */post(a){return this.submit("post","/".concat(this.endpoint),a)}store(a){return this.post(a)}/**
     * Send a PUT request to the given URL.
     *
     * @param {string|int} id
     * @param {Object} item
     */put(a,b){return this.submit("put","/".concat(this.endpoint,"/").concat(a),b)}putWithFile(a,b){return b._method="put",this.submit("post","/".concat(this.endpoint,"/").concat(a),b)}/**
     * Send a PATCH request to the given URL.
     *
     * @param {string|int} id
     * @param {Object} item
     */patch(a,b){return this.submit("patch","/".concat(this.endpoint,"/").concat(a),b)}/**
     * Send a DELETE request to the given URL.
     *
     * @param {string|int} id
     */delete(a){return this.submit("delete","/".concat(this.endpoint,"/").concat(a))}/**
     * Method used to set the query parameters.
     *
     * @param {Object} parameters The given parameters.
     *
     * @returns {BaseProxy} The instance of the proxy.
     */setParameters(a){return Object.keys(a).forEach(b=>{this.parameters[b]=a[b]}),this}/**
     * Method used to set a single parameter.
     *
     * @param {string} parameter The given parameter.
     * @param {*} value The value to be set.
     *
     * @returns {BaseProxy} The instance of the proxy.
     */setParameter(a){var b=1<arguments.length&&void 0!==arguments[1]?arguments[1]:"";if(!b){var c=_qs.default.parse(a,{comma:!0,allowDots:!0,ignoreQueryPrefix:!0});return this.setParameters(c)}return this.parameters[a]=b,this}/**
     * Method used to remove all the parameters.
     *
     * @param {Array} parameters The given parameters.
     *
     * @returns {BaseProxy} The instance of the proxy.
     */removeParameters(){var a=0<arguments.length&&void 0!==arguments[0]?arguments[0]:[];return 0===a.length?this.parameters=[]:a.forEach(a=>{delete this.parameters[a]}),this}/**
     * Method used to remove a single parameter.
     *
     * @param {string} parameter The given parameter.
     *
     * @returns {BaseProxy} The instance of the proxy.
     */removeParameter(a){return delete this.parameters[a],this}/**
     * Submit the form.
     *
     * @param {string} requestType
     * @param {string} url
     * @param {Object|null} form The data that's being send to the API.
     */submit(a,b){var c=2<arguments.length&&void 0!==arguments[2]?arguments[2]:null;return this.__validateRequestType(a),_Validator.default.flush(),_Validator.default.processing=!0,_Validator.default.successful=!1,new Promise((d,e)=>{if(!this.$http)return e(new Error("Axios must be set."));var f=this.hasFiles(c)?(0,_util.objectToFormData)(c):c;this.$http[a](this.getParameterString(b),f).then(a=>{_Validator.default.processing=!1,this.onSuccess(a.data),d(a.data)}).catch(a=>{_Validator.default.processing=!1,_Validator.default.processing=!1;var{response:b}=a||{};if(b){var{data:c,status:d}=b;if(d===UNPROCESSABLE_ENTITY){var f={};Object.assign(f,c.errors),this.onFail(f),_Validator.default.fill(f)}e(a)}else e()})})}/**
     * @returns {boolean}
     */hasFiles(a){for(var b in a){if(!Object.prototype.hasOwnProperty.call(a,b))return!1;if(this.hasFilesDeep(a[b]))return!0}return!1}/**
     * @param {Object|Array} object
     * @returns {boolean}
     */hasFilesDeep(a){if(null===a)return!1;if("object"==typeof a)for(var b in a)if(Object.prototype.hasOwnProperty.call(a,b)&&(0,_util.isFile)(a[b]))return!0;if((0,_util.isArray)(a))for(var c in a)if(Object.prototype.hasOwnProperty.call(a,c))return this.hasFilesDeep(a[c]);return(0,_util.isFile)(a)}/**
     * Handle a successful form submission.
     *
     * @param {object} data
     */onSuccess(){_Validator.default.successful=!0,_Validator.default.flush()}/**
     * Handle a failed form submission.
     *
     * @param {Object} errors
     */onFail(a){_Validator.default.successful=!1,_Validator.default.fill(a)}__validateRequestType(a){var b=["get","delete","head","post","put","patch"];if(!b.includes(a))throw new Error("`".concat(a,"` is not a valid request type, ")+"must be one of: `".concat(b.join("`, `"),"`."))}/**
     * Method used to transform a parameters object to a parameters string.
     *
     * @returns {string} The parameter string.
     */getParameterString(a){var b=_qs.default.stringify(this.parameters,{encode:!1,skipNulls:!0,addQueryPrefix:!0});return"".concat(a).concat(b)}}_defineProperty(BaseProxy,"$http",void 0);var _default=BaseProxy;exports.default=_default;